;**************************************************************
;    Angle (Radians)
;  
;  Authors: Josef Sniatecki
;  Copyright (c) 2010 Josef Sniatecki
;**************************************************************
;GERMAN:
;  Jede Funktion rechnet mit Winkeln im Bogenma
;  engl. radians). Siehe fr Konvertionen von
;  Bogenma zu Grad in "Math.pbi".
;ENGLISH:
;  Each function works with angles in radians.
;  For convertions look at "Math.pbi".



CompilerIf Defined(Angle_PBI, #PB_Constant) = #False
#Angle_PBI = #True



;******************************************
; Functions
;******************************************

Procedure.f Angle(Angle.f)
  ;  Author: Josef Sniatecki
  ;
  ;DEUTSCH:
  ;  Gibt den Winkel "Angle" zwischen "0" und "#PI" zurck.
  
  If Angle < 0
    ProcedureReturn #PI*2 + Angle - Int(Angle/(#PI*2))*2*#PI
  ElseIf Angle => 2*#PI
    ProcedureReturn Angle - Int(Angle/(2*#PI))*2*#PI
  EndIf
  ProcedureReturn Angle 
EndProcedure
Procedure.f AngleDif(Angle1.f, Angle2.f)
  ;  Author: Josef Sniatecki
  ;  Thanks to STARGATE (http://www.pure-board.de/):
  ;
  ;DEUTSCH:
  ;  Berechnet die Differenz zwischen den zwei Winkeln
  ;  "Angle1" und "Angle2". Dabei wird die kleinste
  ;  Differenz zurckgegeben.
  
  Protected Difference.f
  
  Difference = Angle2 - Angle1
  If Difference > #PI
    ProcedureReturn Difference - #PI*2
  ElseIf Difference < -#PI
    ProcedureReturn Difference + #PI*2
  Else
    ProcedureReturn Difference
  EndIf
EndProcedure
Procedure.f AngleTo(SX.f, SY.f, DX.f, DY.f)
  ;  Author: Josef Sniatecki
  ;
  ;DEUTSCH:
  ;  Gibt den Winkel von Punkt S ("SX"|"SY") zu Punkt
  ;  D ("DX"|"DY") zurck.
  
  Protected Angle.f
  Protected XDis.f
  Protected YDis.f
  
  XDis  = DX - SX
  YDis  = DY - SY
  Angle = ACos(XDis / Sqr(XDis*XDis + YDis*YDis))
  
  ; Fallunterscheidung:
  If SY > DY
    Angle = -Angle
  EndIf
  
  ProcedureReturn Angle
EndProcedure



CompilerEndIf

; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 23
; FirstLine = 16
; Folding = -
; EnableXP