;**************************************************************
;     Button
;
;  Authors: Josef Sniatecki
;
;  Copyright (c) 2010 Josef Sniatecki
;**************************************************************



CompilerIf Defined(Button_PBI, #PB_Constant) = #False
#Button_PBI = #True



;******************************************
; Includes
;******************************************

IncludeFile #PB_Compiler_FilePath + "Standard.pbi"
IncludeFile #PB_Compiler_FilePath + "Window.pbi"



;******************************************
; Constants
;******************************************

Enumeration
  #Button_Normal
  #Button_MouseOver
  #Button_Push
EndEnumeration



;******************************************
; Structures
;******************************************

Structure Button
  Sprite.i
  State.i
  X.i
  Y.i
  Width.i
  Height.i
  Visible.i
EndStructure



;******************************************
; Variables
;******************************************

Global *Button_Array
Global Button_MaxIndex.i
Global *Button_Current.Button
Global Button_Pushed.i



;******************************************
; Functions
;******************************************

Procedure.i UseButtons(Array Button.Button(1))
  *Button_Array   = @Button(0)
  Button_MaxIndex = ArraySize(Button())
  *Button_Current = 0
EndProcedure
Procedure.i SetButton(Index.i, Sprite.i, X.i, Y.i, Visible.i = #True)
  Protected *Button.Button = *Button_Array + Index * _Sz(Button)
  
  *Button\Sprite  = Sprite
  *Button\State   = #Button_Normal
  *Button\X       = X
  *Button\Y       = Y
  *Button\Width   = SpriteWidth(Sprite) / 3
  *Button\Height  = SpriteHeight(Sprite)
  *Button\Visible = Visible
EndProcedure

Procedure.i ExamineButtons(Event.i, MouseX.i, MouseY.i)
  Protected *Button.Button, Index.i
  
  If *Button_Current
    *Button = *Button_Current
    Select *Button\State
      Case #Button_MouseOver
        If Not (MouseX => *Button\X And MouseX < *Button\X + *Button\Width And MouseY => *Button\Y And MouseY < *Button\Y + *Button\Height)
          *Button_Current = #Null
          *Button\State = #Button_Normal
        ElseIf Event = #PB_Event_LeftMouseButtonDown
          *Button\State = #Button_Push
        ElseIf Event = #PB_Event_LeftDoubleClick
          *Button\State   = #Button_Push
        EndIf
      Case #Button_Push
        If Event = #PB_Event_LeftMouseButtonUp
          If MouseX => *Button\X And MouseX < *Button\X + *Button\Width And MouseY => *Button\Y And MouseY < *Button\Y + *Button\Height
            *Button\State   = #Button_MouseOver
            Button_Pushed   = (*Button - *Button_Array) / _Sz(Button)
            ProcedureReturn #True
          Else
            *Button_Current = #Null
            *Button\State   = #Button_Normal
          EndIf
        EndIf
    EndSelect
  Else
    *Button = *Button_Array
    For Index = 0 To Button_MaxIndex
      If *Button\Visible
        If MouseX => *Button\X And MouseX < *Button\X + *Button\Width And MouseY => *Button\Y And MouseY < *Button\Y + *Button\Height
          *Button\State = #Button_MouseOver
          *Button_Current = *Button
          Break
        EndIf
      EndIf
      *Button + _Sz(Button)
    Next
  EndIf
  
  ProcedureReturn #False
EndProcedure
Procedure.i DisplayButtons()
  Protected *Button.Button = *Button_Array, Index.i
  
  For Index = 0 To Button_MaxIndex
    If *Button\Visible
      ClipSprite(*Button\Sprite, *Button\State * *Button\Width, 0, *Button\Width, *Button\Height)
      DisplaySprite(*Button\Sprite, *Button\X, *Button\Y)
    EndIf
    *Button + _Sz(Button)
  Next
EndProcedure
Procedure.i DisplayTransparentButtons()
  Protected *Button.Button = *Button_Array, Index.i
  
  For Index = 0 To Button_MaxIndex
    If *Button\Sprite
      ClipSprite(*Button\Sprite, *Button\State * *Button\Width, 0, *Button\Width, *Button\Height)
      DisplayTransparentSprite(*Button\Sprite, *Button\X, *Button\Y)
    EndIf
    *Button + _Sz(Button)
  Next
EndProcedure
Procedure.i PushedButton()
  ProcedureReturn Button_Pushed
EndProcedure
Procedure.i ActiveButton()
  ProcedureReturn *Button_Current
EndProcedure




;******************************************
; Example
;******************************************

; IncludeFile "Screen.pbi"
; 
; 
; 
; Global Sprite.i
; Global Dim Button.Button(5)
; Global PreviousWidth.i, PreviousHeight.i, PreviousBitsPerPel.i, PreviousFrequency.i
; 
; 
; 
; InitSprite()
; 
; GetScreenSettings(@PreviousWidth, @PreviousHeight, @PreviousBitsPerPel, @PreviousFrequency)
; SetScreenSettings(1024, 768, 32, 60) ;Wenn ntig anpassen!
; OpenWindow(0, 0, 0, 0, 0, "Test", #PB_Window_Maximize|#PB_Window_BorderLess)
; OpenWindowedScreen(WindowID(0), 0, 0, 1024, 768, 0, 0, 0) ;Hier ebenso anpassen, wenn ntig!
; 
; 
; 
; Sprite = CreateSprite(#PB_Any, 80 * 3, 30)
; StartDrawing(SpriteOutput(Sprite))
;   Box(0  , 0, 80, 30, $C0C0C0) ;Normal
;   Box(80 , 0, 80, 30, $FF0000) ;Mouse over
;   Box(160, 0, 80, 30, $800000) ;Push
; StopDrawing()
; 
; 
; 
; UseButtons(Button())
; For I = 0 To 5
;   SetButton(I, Sprite, 200, 200 + I*40)
; Next
; 
; Repeat
;   Event = WindowEvent()
;   
;   ClearScreen(#Black)
;     ; Buttons anzeigen:
;     DisplayButtons()
;   FlipBuffers()
;   
;   ; Events fr Buttons abfragen:
;   If ExamineButtons(Event, WindowMouseX(0), WindowMouseY(0))
;     ; Button wurde gedrckt:
;     Debug "Button nr. " + Str(PushedButton() + 1) + " pushed"
;   Else
;     ; Sonstige Abfragen:
;     Select Event
;       Case #PB_Event_KeyDown
;         If EventwParam() = #VK_ESCAPE
;           Break
;         EndIf
;     EndSelect
;   EndIf
;   
;   Delay(1)
; ForEver
; 
; CloseWindow(0)
; SetScreenSettings(PreviousWidth, PreviousHeight, PreviousBitsPerPel, PreviousFrequency)
; 
; End

      

CompilerEndIf

; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 205
; FirstLine = 191
; Folding = --
; EnableXP