;**************************************************************
;    Collision (2D)
;  
;  Authors: Josef Sniatecki
;  Copyright (c) 2010 Josef Sniatecki
;**************************************************************



CompilerIf Defined(Collision_PBI, #PB_Constant) = #False
#Collision_PBI = #True



;******************************************
; Constants
;******************************************

#Collision_Left   = %0001
#Collision_Right  = %0010
#Collision_Top    = %0100
#Collision_Bottom = %1000



;******************************************
; Functions
;******************************************

Procedure.i RectangleOverlap(X1.f, Y1.f, Width1.f, Height1.f, X2.f, Y2.f, Width2.f, Height2.f)
  ;  Author: Josef Sniatecki
  
  If X1 + Width1 > X2 And X1 < X2 + Width2 And Y1 + Height1 > Y2 And Y1 < Y2 + Height2
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i RectangleCollision(X1.f, Y1.f, Width1.f, Height1.f, X2.f, Y2.f, Width2.f, Height2.f)
  ;  Author: Josef Sniatecki
  
  Protected A.f, B.f
 
  A = (X2 + Width2/2)  - (X1 + Width1/2)
  B = (Y2 + Height2/2) - (Y1 + Height1/2)
 
  If Abs(A) - (Width1/2) - (Width2/2) < 0 And Abs(B) - (Height1/2) - (Height2/2) < 0
    If A => 0
      If B => 0
        If Abs(A) - (Width1/2) - (Width2/2) > Abs(B) - (Height1/2) - (Height2/2)
          ProcedureReturn #Collision_Left
        Else
          ProcedureReturn #Collision_Top
        EndIf
      Else
        If Abs(A) - (Width1/2) - (Width2/2) > Abs(B) - (Height1/2) - (Height2/2)
          ProcedureReturn #Collision_Left
        Else
          ProcedureReturn #Collision_Bottom
        EndIf
      EndIf
    Else
      If B => 0
        If Abs(A) - (Width1/2) - (Width2/2) > Abs(B) - (Height1/2) - (Height2/2)
          ProcedureReturn #Collision_Right
        Else
          ProcedureReturn #Collision_Top
        EndIf
      Else
        If Abs(A) - (Width1/2) - (Width2/2) > Abs(B) - (Height1/2) - (Height2/2)
          ProcedureReturn #Collision_Right
        Else
          ProcedureReturn #Collision_Bottom
        EndIf
      EndIf
    EndIf
  Else
    ProcedureReturn #False
  EndIf
EndProcedure

Procedure.i OutOfRectangle(X1.f, Y1.f, Width1.f, Height1.f, X2.f, Y2.f, Width2.f, Height2.f)
  ;  Author: Josef Sniatecki
  
  Protected Collision.i
  
  If X1 < X2
    Collision & #Collision_Left
  ElseIf X1 + Width1 > X2 + Width2
    Collision & #Collision_Right
  EndIf
  
  If Y1 < Y2
    Collision & #Collision_Top
  ElseIf Y1 + Height1 > Y2 + Height2
    Collision & #Collision_Bottom
  EndIf
  
  ProcedureReturn Collision
EndProcedure



CompilerEndIf

; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 83
; FirstLine = 69
; Folding = -
; EnableXP