;**************************************************************
;    Cursor
;  
;  Authors: Josef Sniatecki
;  Copyright (c) 2010 Josef Sniatecki
;**************************************************************



CompilerIf Defined(Cursor_PBI, #PB_Constant) = #False
#Cursor_PBI = #True



;******************************************
; Constants
;******************************************

#Mouse_LeftButtonDown   = #MOUSEEVENTF_LEFTDOWN
#Mouse_LeftButtonUp     = #MOUSEEVENTF_LEFTUP
#Mouse_RightButtonDown  = #MOUSEEVENTF_RIGHTDOWN
#Mouse_RightButtonUp    = #MOUSEEVENTF_RIGHTUP
#Mouse_MiddleButtonDown = #MOUSEEVENTF_MIDDLEDOWN
#Mouse_MiddleButtonUp   = #MOUSEEVENTF_MIDDLEUP



;******************************************
; Functions
;******************************************

CompilerSelect #PB_Compiler_OS
  CompilerCase #PB_OS_Windows
    Procedure.i CursorY() ;Returns the actual cursor X position on the desktop.
      Protected Position.POINT
      
      GetCursorPos_(@Position)
      ProcedureReturn Position\X
    EndProcedure
    Procedure.i CursorX() ;Returns the actual cursor Y position on the desktop. 
      Protected Position.POINT
      
      GetCursorPos_(@Position)
      ProcedureReturn Position\Y
    EndProcedure
    Procedure.i SetCursorX(X.i) ;Set the actual cursor X on the desktop.
      SetCursorPos_(X, -1)
    EndProcedure
    Procedure.i SetCursorY(Y.i) ;Set the actual cursor X on the desktop.
      SetCursorPos_(-1, Y)
    EndProcedure
    Procedure.i CursorLocate(X.i, Y.i) ;Locate the cursor position on the desktop.
      SetCursorPos_(X, Y)
    EndProcedure
    Procedure.i MoveCursor(DestinationX.i, DestinationY.i) ;Move the cursor on the desktop.
      Protected Position.POINT
      
      GetCursorPos_(@Position)
      SetCursorPos_(Position\X + DestinationX, Position\Y + DestinationY)
    EndProcedure
    
    Procedure.i ClickMouseButton(Button.i)
      mouse_event_(Button, 0, 0, 0, 0)
    EndProcedure
    Procedure.i RotateMouseWheel(Ticks.i)
      mouse_event_(#MOUSEEVENTF_WHEEL, 0, 0, Ticks * 120, 0)
    EndProcedure
    
    Procedure.i CursorOverGadget(Window.i, Gadget.i)
      Protected Position.POINT
      
      Position\X = WindowMouseX(Window)
      Position\Y = WindowMouseY(Window)
      
      If Position\X => GadgetX(Gadget) And Position\X < GadgetX(Gadget) + GadgetWidth(Gadget) And Position\Y => GadgetY(Gadget) And Position\Y < GadgetY(Gadget) + GadgetHeight(Gadget)
        ProcedureReturn #True
      Else
        ProcedureReturn #False
      EndIf
    EndProcedure
  CompilerDefault
    CompilerError "Only supported under Windows."
CompilerEndSelect



CompilerEndIf

; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 78
; FirstLine = 54
; Folding = --
; EnableXP