;**************************************************************
;    Editor gadget (extension for RTF)
;  
;  Authors: Josef Sniatecki
;  Copyright (c) 2010 Josef Sniatecki
;**************************************************************



CompilerIf Defined(EditorGadget_PBI, #PB_Constant) = #False
#EditorGadget_PBI = #True

CompilerIf #PB_Compiler_OS <> #PB_OS_Windows
  CompilerError "Only supported under Windows."
CompilerEndIf



;******************************************
; Constants
;******************************************

#PB_EditorGadget_Bold      = #CFM_BOLD
#PB_EditorGadget_Italic    = #CFM_ITALIC
#PB_EditorGadget_Underline = #CFM_UNDERLINE
#PB_EditorGadget_Strikeout = #CFM_STRIKEOUT

#PB_FindText_CaseSensitive = #FR_MATCHCASE
#PB_FindText_WholeWord     = #FR_WHOLEWORD



;******************************************
; Functions
;******************************************

Procedure.i EditorGadgetSelect(Gadget.i, StartIndex.i, EndIndex.i)
  Protected Range.CHARRANGE
  
  Range\cpMin = StartIndex
  Range\cpMax = EndIndex
  SendMessage_(GadgetID(Gadget), #EM_EXSETSEL, 0, @Range)
EndProcedure
Procedure.i EditorGadgetSelectXY(Gadget.i, StartLine.i, StartChar.i = 0, EndLine.i = -1, EndChar.i = -1)
  Protected Range.CHARRANGE, GadgetID.i = GadgetID(Gadget)
  
  Range\cpMin = SendMessage_(GadgetID, #EM_LINEINDEX, StartLine, 0) + StartChar - 1
  
  If EndLine = -1
    EndLine = SendMessage_(GadgetID, #EM_GETLINECOUNT, 0, 0) - 1
  EndIf
  
  Range\cpMax = SendMessage_(GadgetID, #EM_LINEINDEX, EndLine, 0)
  
  If EndChar = -1
    Range\cpMax + SendMessage_(GadgetID, #EM_LINELENGTH, Range\cpMax, 0)
  Else
    Range\cpMax + EndChar - 1
  EndIf
  
  SendMessage_(GadgetID, #EM_EXSETSEL, 0, @Range)
EndProcedure
Procedure.i EditorGadgetSelectAll(Gadget.i)
  ;ENGLISH:
  ;  Select the whole content of the editor gadget.
  
  Protected Range.CHARRANGE
  
  Range\cpMin = 0 
  Range\cpMax = -1 
  SendMessage_(GadgetID(Gadget), #EM_EXSETSEL, 0, @Range)
EndProcedure
Procedure.i EditorGadgetGetSelection(Gadget.i, *Selection.CHARRANGE)
  ;ENGLISH:
  ;  Get the range of the current selection.
  
  SendMessage_(GadgetID(Gadget), #EM_EXGETSEL, 0, *Selection)
EndProcedure
Procedure.s EditorGadgetGetSelectedText(Gadget.i)
  Protected Text.s, Range.CHARRANGE, GadgetID.i = GadgetID(Gadget)
  
  SendMessage_(GadgetID, #EM_EXGETSEL, 0, @Range)
  Text = Space(Range\cpMax - Range\cpMin)
  SendMessage_(GadgetID, #EM_GETSELTEXT, 0, @Text)
  
  ProcedureReturn Text
EndProcedure
Procedure.i EditorGadgetReplaceSelection(Gadget.i, Text.s)
  SendMessage_(GadgetID(Gadget), #EM_REPLACESEL, 0, @Text)
EndProcedure

Procedure.i EditorGadgetCursorX(Gadget.i)
  Protected GadgetID.i = GadgetID(Gadget), Range.CHARRANGE
  
  SendMessage_(GadgetID, #EM_EXGETSEL, 0, @Range)
  ProcedureReturn (Range\cpMax - (SendMessage_(GadgetID, #EM_LINEINDEX, SendMessage_(GadgetID, #EM_EXLINEFROMCHAR, 0, Range\cpMin), 0)) + 1)
EndProcedure 
Procedure.i EditorGadgetCursorY(Gadget.i) 
  Protected GadgetID.i = GadgetID(Gadget), Range.CHARRANGE
  
  SendMessage_(GadgetID, #EM_EXGETSEL, 0, @Range)
  ProcedureReturn SendMessage_(GadgetID, #EM_EXLINEFROMCHAR, 0, Range\cpMin) + 1 
EndProcedure 
Procedure.i EditorGadgetLines(Gadget.i)
  ProcedureReturn SendMessage_(GadgetID(Gadget), #EM_GETLINECOUNT, 0, 0)
EndProcedure

Procedure.i EditorGadgetSetStyle(Gadget.i, Style.i)
  ;ENGLISH:
  ;  Modify the font style of the selection.
  
  Protected Format.CHARFORMAT
  
  Format\cbSize    = SizeOf(CHARFORMAT)
  Format\dwMask    = #CFM_ITALIC|#CFM_BOLD|#CFM_STRIKEOUT|#CFM_UNDERLINE
  Format\dwEffects = Style
  
  SendMessage_(GadgetID(Gadget), #EM_SETCHARFORMAT, #SCF_SELECTION, @Format)
EndProcedure
Procedure.i EditorGadgetSetColor(Gadget.i, Color.i)
  ;ENGLISH:
  ;  Set the foreground color of the current selection.
  
  Protected Format.CHARFORMAT
  
  Format\cbSize      = SizeOf(CHARFORMAT)
  Format\dwMask      = #CFM_COLOR
  Format\crTextColor = Color
  
  SendMessage_(GadgetID(Gadget), #EM_SETCHARFORMAT, #SCF_SELECTION, @Format)
EndProcedure
Procedure.i EditorGadgetSetFont(Gadget.i, Name.s)
  Protected Format.CHARFORMAT
  
  Format\cbSize = SizeOf(CHARFORMAT)
  Format\dwMask = #CFM_FACE
  PokeS(@Format\szFaceName, Name)
  
  SendMessage_(GadgetID(Gadget), #EM_SETCHARFORMAT, #SCF_SELECTION, @Format)
EndProcedure
Procedure.i EditorGadgetSetFontSize(Gadget.i, Size.i)
  Protected Format.CHARFORMAT
  
  Format\cbSize  = SizeOf(CHARFORMAT)
  Format\dwMask  = #CFM_SIZE
  Format\yHeight = Size * 20
  
  SendMessage_(GadgetID(Gadget), #EM_SETCHARFORMAT, #SCF_SELECTION, @Format)
EndProcedure

Procedure.i EditorGadgetInsertText(Gadget.i, Text.s)
  ;ENGLISH:
  ;  Insert a text at the current position/selection.
  
  SendMessage_(GadgetID(Gadget), #EM_REPLACESEL, 1 , @Text)
EndProcedure
Procedure.i EditorGadgetAppendText(Gadget.i, Text.s)
  ;ENGLISH:
  ;  Insert a text at the end of the whole text.
  
  Protected GadgetID.i = GadgetID(Gadget)
  
  SendMessage_(GadgetID, #EM_SETSEL    , -1, -1)
  SendMessage_(GadgetID, #EM_REPLACESEL, 1 , @Text)
EndProcedure

Procedure.i EditorGadgetCut(Gadget.i)
  SendMessage_(GadgetID(Gadget), #WM_CUT, 0, 0)
EndProcedure
Procedure.i EditorGadgetCopy(Gadget.i)
  SendMessage_(GadgetID(Gadget), #WM_COPY, 0, 0)
EndProcedure
Procedure.i EditorGadgetPaste(Gadget.i)
  SendMessage_(GadgetID(Gadget), #WM_PASTE, 0, 0)
EndProcedure
Procedure.i EditorGadgetUndo(Gadget.i)
  SendMessage_(GadgetID(Gadget), #EM_UNDO, 0, 0)
EndProcedure
Procedure.i EditorGadgetRedo(Gadget.i)
  SendMessage_(GadgetID(Gadget), #EM_REDO, 0, 0)
EndProcedure

Procedure.i EditorGadgetFindText(Gadget.i, Text.s, Flags.i = 0, Selection.i = #True)
  ;DEUTSCH:
  ;  Falls eine Adresse fr "*Info" angegeben wird, so
  ;  sucht diese Funktion ab der aktuellen Position.
  
  Protected GadgetID.i = GadgetID(Gadget), Found.i, Info.FINDTEXT, Range.CHARRANGE
  
  SendMessage_(GadgetID, #EM_EXGETSEL, 0, @Range)
  Info\chrg\cpMin = Range\cpMax
  Info\chrg\cpMax = -1
  Info\lpstrText  = @Text
  
  SendMessage_(GadgetID, #EM_SETSEL, 0, 0)
  
  Found = SendMessage_(GadgetID, #EM_FINDTEXT, Flags|#FR_DOWN, @Info)
  If Found > -1
    Info\chrg\cpMin = Found + 1
    If Selection
      SendMessage_(GadgetID, #EM_SETSEL, Found, Found + Len(Text))
    EndIf
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i EditorGadgetReplaceAll(Gadget.i, ToReplace.s, Text.s, Flags.i = 0)
  Protected GadgetID.i = GadgetID(Gadget), Info.FINDTEXT, TextLength.i = Len(ToReplace)
  Protected Count.i, Range.CHARRANGE, Found.i
  
  Info\lpstrText  = @ToReplace
  Info\chrg\cpMin = 0
  Info\chrg\cpMax = -1
  
  Flags|#FR_DOWN
  
  SendMessage_(GadgetID, #EM_SETSEL, 0, 0)
  
  Repeat
    Found = SendMessage_(GadgetID, #EM_FINDTEXT, Flags, @Info)
    If Found > -1
      Info\chrg\cpMin = Found + 1
      
      Range\cpMin = Found
      Range\cpMax = Found + TextLength
      SendMessage_(GadgetID, #EM_EXSETSEL, 0, @Range)
      SendMessage_(GadgetID, #EM_REPLACESEL, 0, @Text)
      Count + 1
    Else
      ProcedureReturn Count
    EndIf
  ForEver
EndProcedure

Procedure.i EditorGadgetModified(Gadget.i)
  Protected GadgetID.i = GadgetID(Gadget)
  
  If SendMessage_(GadgetID, #EM_GETMODIFY, 0, 0)
    SendMessage_(GadgetID, #EM_SETMODIFY, 0, 0)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure

Procedure.i EditorGadgetLoad_Callback(File.i, Buffer.i, CB.i, PCB.i)
  ProcedureReturn ReadFile_(File, Buffer, CB, PCB, 0) ! 1
EndProcedure
Procedure.i EditorGadgetLoad(Gadget.i, FileName.s)
  ;  Author: Josef Sniatecki
  ;  Thanks to Thorsten (www.pure-board.de) and El_Choni (http://www.purebasic.fr/english/)!
  
  Protected StreamData.EDITSTREAM, File.i
  
  File = ReadFile(#PB_Any, FileName)
  If File
    StreamData\dwCookie    = FileID(File)
    StreamData\dwError     = #Null
    StreamData\pfnCallback = @EditorGadgetLoad_Callback()
    
    SendMessage_(GadgetID(Gadget), #EM_STREAMIN, #SF_TEXT, @StreamData) 
    
    CloseFile(File)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure 
Procedure.i EditorGadgetSave_Callback(File.i, Buffer.i, CB.i, PCB.i)
  ProcedureReturn WriteFile_(File, Buffer, CB, PCB, 0) ! 1
EndProcedure
Procedure.i EditorGadgetSave(Gadget.i, FileName.s)
  ;  Author: Josef Sniatecki
  ;  Thanks to Thorsten (www.pure-board.de) and El_Choni (http://www.purebasic.fr/english/)!
  
  Protected StreamData.EDITSTREAM, File.i
    
  File = CreateFile(#PB_Any, FileName)
  If File
    StreamData\dwCookie    = FileID(File)
    StreamData\dwError     = #Null
    StreamData\pfnCallback = @EditorGadgetSave_Callback()
    
    SendMessage_(GadgetID(Gadget), #EM_STREAMOUT, #SF_TEXT, @StreamData)
    
    CloseFile(File)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure 
 


CompilerEndIf

; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 16
; Folding = AAAA5
; EnableXP