;**************************************************************
;    Function plotter
;  
;  Authors: Josef Sniatecki
;  Copyright (c) 2010 Josef Sniatecki
;**************************************************************
;DEUTSCH:
;  Ermglicht das Zeichnen von Funktionen auf Images.



CompilerIf Defined(FunctionPlotter_PBI, #PB_Constant) = #False
#FunctionPlotter_PBI = #True

CompilerIf #PB_Compiler_OS <> #PB_OS_Windows
  ;Diese Quelldatei verwendet das Erstellen von Pens, was
  ;mir nur in Windows durch "CreatePen_" bekannt ist.
  
  CompilerError "Only supported under Windows."
CompilerEndIf



;******************************************
; Includes
;******************************************

IncludeFile #PB_Compiler_FilePath + "Standard.pbi"



;******************************************
; Structures
;******************************************

Structure Plotter
  Image.i
  AX.i
  BX.i
  AY.i
  BY.i
  Width.i
  Height.i
  Unit.i
EndStructure



;******************************************
; Variables
;******************************************

Prototype.f Plotter_F(X.f)



;******************************************
; Functions
;******************************************

Procedure.i CreatePlotter(Image.i, AX.i, AY.i, Unit.i = 10)
  Protected *Plotter.Plotter
  
  *Plotter = _New(Plotter)
  
  *Plotter\Image     = Image
  *Plotter\AX        = AX
  *Plotter\BX        = AX + (ImageWidth(Image) / Unit)
  *Plotter\AY        = AY
  *Plotter\BY        = AY + (ImageHeight(Image) / Unit)
  *Plotter\Width     = ImageWidth(Image)
  *Plotter\Height    = ImageHeight(Image)
  *Plotter\Unit      = Unit
  
  ProcedureReturn *Plotter
EndProcedure
Procedure.i PlotterDrawBackground(*Plotter.Plotter, BackgroundColor.i = $FFFFFF, GridColor.i = $D0D0D0, AxeColor.i = $808080)
  Protected Index.i, Length.i
  
  StartDrawing(ImageOutput(*Plotter\Image))
    Box(0, 0, *Plotter\Width, *Plotter\Height, BackgroundColor)
    
    Length = *Plotter\Width / *Plotter\Unit
    For Index = 1 To Length
      Line(Index * *Plotter\Unit - 1, 0, 1, *Plotter\Height, GridColor)
    Next
    Length = *Plotter\Height / *Plotter\Unit
    For Index = 1 To Length
      Line(0, Index * *Plotter\Unit - 1, *Plotter\Width, 1, GridColor)
    Next
    
    If *Plotter\AX < 0 And -*Plotter\AX < *Plotter\Width
      Line(-*Plotter\AX * *Plotter\Unit - 1, 0, 1, *Plotter\Height, AxeColor)
    EndIf
    If *Plotter\AY > 0 And *Plotter\AY < *Plotter\Height
      Line(0 ,*Plotter\AY * *Plotter\Unit - 1, *Plotter\Width, 1, AxeColor)
    EndIf
  StopDrawing()
EndProcedure
Procedure.i PlotterDraw(*Plotter.Plotter, *Function, Color.i = $FF0000, Thickness.i = 1)
  Protected X.f, PreviousX.f, Steps.f, Y.f, PreviousY.f
  Protected F.Plotter_F, AX.i = *Plotter\AX, AY.i = *Plotter\AY, Unit = *Plotter\Unit
  Protected DC.i, Pen.i, PreviousPen.i, Distance.i
  
  Steps     = 1.0 / Unit
  PreviousX = AX
  X         = PreviousX + Steps
  Distance  = *Plotter\Height / Unit
  
  PokeI(@F, *Function)
  
  DC = StartDrawing(ImageOutput(*Plotter\Image))
    If Thickness
      Pen = CreatePen_(#PS_SOLID, Thickness, Color)
    Else
      Pen = CreatePen_(#PS_DOT, 1, Color)
    EndIf
    PreviousPen = SelectObject_(DC, Pen)
    
    PreviousY = F(PreviousX)
    While X < *Plotter\BX
      Y = F(X)
      
      If Abs(Y - PreviousY) < Distance
        MoveToEx_(DC, Int((PreviousX - AX) * Unit), Int((-PreviousY + AY) * Unit), #Null)
        LineTo_(DC, Int((X - AX) * Unit), Int((-Y + AY) * Unit))
      EndIf
      
      PreviousX = X
      PreviousY = Y
      
      X + Steps
    Wend
    
    DeleteObject_(Pen)
    SelectObject_(DC, PreviousPen)
  StopDrawing()
EndProcedure



;******************************************
; Using Operno
;******************************************
;DEUTSCH:
;  Falls Operno ("Operno.pbi") eingebunden wurde, knnen auch
;  Funktionen mit zur Laufzeit definierten Funktionstermen
;  gezeichnet werden.

CompilerIf Defined(Operno_PBI, #PB_Constant)
  
  ; Include hyperbolic functions:
  IncludeFile #PB_Compiler_FilePath + "HyperbolicFunctions.pbi"
  
  Global *Plotter_Object.OpernoObject
  Global *Plotter_X.OpernoObject
  Global *Plotter_X_Float.Float
  
  ; Define functions compatible with Operno:
  Procedure.i Plotter_Sin()
    Protected X.f
    
    OpernoPopFValue(@X)
    ProcedureReturn OpernoPushFValue(Sin(X))
  EndProcedure
  Procedure.i Plotter_Cos()
    Protected X.f
    
    OpernoPopFValue(@X)
    ProcedureReturn OpernoPushFValue(Cos(X))
  EndProcedure
  Procedure.i Plotter_Tan()
    Protected X.f
    
    OpernoPopFValue(@X)
    ProcedureReturn OpernoPushFValue(Tan(X))
  EndProcedure
  Procedure.i Plotter_Cot()
    Protected X.f
    
    OpernoPopFValue(@X)
    ProcedureReturn OpernoPushFValue(Cot(X))
  EndProcedure
  Procedure.i Plotter_SinH()
    Protected X.f
    
    OpernoPopFValue(@X)
    ProcedureReturn OpernoPushFValue(SinH(X))
  EndProcedure
  Procedure.i Plotter_CosH()
    Protected X.f
    
    OpernoPopFValue(@X)
    ProcedureReturn OpernoPushFValue(CosH(X))
  EndProcedure
  Procedure.i Plotter_TanH()
    Protected X.f
    
    OpernoPopFValue(@X)
    ProcedureReturn OpernoPushFValue(TanH(X))
  EndProcedure
  Procedure.i Plotter_CotH()
    Protected X.f
    
    OpernoPopFValue(@X)
    ProcedureReturn OpernoPushFValue(CotH(X))
  EndProcedure
  
  Procedure.i PlotterDrawExpression(*Plotter.Plotter, Expression.s, Color.i = $FF0000, Thickness.i = 1)
    Protected X.f, PreviousX.f, Steps.f, Y.f, PreviousY.f
    Protected *OpernoCode, AX.f = *Plotter\AX, AY.f = *Plotter\AY, Unit.f = *Plotter\Unit
    Protected DC.i, Pen.i, PreviousPen.i, Distance.i
    
    Steps     = 1.0 / Unit
    PreviousX = AX
    X         = PreviousX + Steps
    Distance  = *Plotter\Height / Unit
    
    OpernoUseObject(*Plotter_Object)
    *OpernoCode = OpernoParse(Expression)
    If OpernoError()
      OpernoCloseLevel()
      ProcedureReturn #False
    EndIf
    
    DC = StartDrawing(ImageOutput(*Plotter\Image))
      If Thickness
        Pen = CreatePen_(#PS_SOLID, Thickness, Color)
      Else
        Pen = CreatePen_(#PS_DOT, 1, Color)
      EndIf
      PreviousPen = SelectObject_(DC, Pen)
      
      *Plotter_X_Float\f = PreviousX
      PreviousY = OpernoRunF(*OpernoCode)
      
      While X < *Plotter\BX
        *Plotter_X_Float\f = X
        Y = OpernoRunF(*OpernoCode)
        OpernoClearStack()
        
        If OpernoError()
          OpernoCloseLevel()
          ProcedureReturn #False
        EndIf
        
        If Abs(Y - PreviousY) < Distance
          MoveToEx_(DC, Int((PreviousX - AX) * Unit), Int((-PreviousY + AY) * Unit), #Null)
          LineTo_(DC, Int((X - AX) * Unit), Int((-Y + AY) * Unit))
        EndIf
        
        PreviousX = X
        PreviousY = Y
        
        X + Steps
      Wend
      
      DeleteObject_(Pen)
      SelectObject_(DC, PreviousPen)
    StopDrawing()
    
    OpernoCloseLevel()
    OpernoFreeTokenQueue(*OpernoCode)
    
    ProcedureReturn #True
  EndProcedure
  
  Procedure.i InitExpressionPlotter()
    *Plotter_Object = OpernoCreateGroup("plotter")
    OpernoExtractObject(*Plotter_Object, ?Plotter_Object)
    
    *Plotter_X       = OpernoCreateFVariable("x", 0.0, *Plotter_Object)
    *Plotter_X_Float = OpernoVariableMemory(*Plotter_X)
  EndProcedure
  
  DataSection
    Plotter_Object:
      Data.c #OpernoObject_Function : Data.s "sin" ;{
        Data.i @Plotter_Sin()
      ;}
      Data.c #OpernoObject_Function : Data.s "cos" ;{
        Data.i @Plotter_Cos()
      ;}
      Data.c #OpernoObject_Function : Data.s "tan" ;{
        Data.i @Plotter_Tan()
      ;}
      Data.c #OpernoObject_Function : Data.s "cot" ;{
        Data.i @Plotter_Cot()
      ;}
      Data.c #OpernoObject_Function : Data.s "sinh" ;{
        Data.i @Plotter_SinH()
      ;}
      Data.c #OpernoObject_Function : Data.s "cosh" ;{
        Data.i @Plotter_CosH()
      ;}
      Data.c #OpernoObject_Function : Data.s "tanh" ;{
        Data.i @Plotter_TanH()
      ;}
      Data.c #OpernoObject_Function : Data.s "coth" ;{
        Data.i @Plotter_CotH()
      ;}
      Data.c #Null
  EndDataSection
  
CompilerEndIf



CompilerEndIf

; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 158
; FirstLine = 143
; Folding = ----
; EnableXP