;**************************************************************
;    Math
;  
;  Authors: Josef Sniatecki
;  Copyright (c) 2010 Josef Sniatecki
;**************************************************************



CompilerIf Defined(Math_PBI, #PB_Constant) = #False
#Math_PBI = #True



;******************************************
; Macros
;******************************************

Macro _Rad(Degrees)
  ((Degrees) / 180 * #PI)
EndMacro
Macro _Deg(Radians)
  ((Radians) * 180 / #PI)
EndMacro



;******************************************
; Constants
;******************************************

#E    = 2.7182818284590452354
#SQR2 = 1.4142135623730950488
#SQR3 = 1.7320508075688772935
#SA   = 1.6180339887498948482 ;Goldener Schnitt


;******************************************
; Functions
;******************************************

; Numberic:
Procedure.i QSum(Number.q) ;Returns the square sum.
  Protected Result.i
  
  While Number
    Result + Number % 10
    Number / 10
  Wend
  ProcedureReturn Result
EndProcedure
Procedure.i GCD(Number1.i, Number2.i) ;Returns the greatest common divisor of number1 and number2.
  Protected GGT.i
  
  Repeat
    GGT = Number1 % Number2
    If GGT
      Number1 = Number2
      Number2 = GGT
    EndIf
  Until Not GGT
  ProcedureReturn Number2
EndProcedure
Procedure.i LCD(Number1.i, Number2.i) ;Returns the lowest common divisor of number1 and number2.
  ProcedureReturn Number1 * Number2 / GCD(Number1, Number2)
EndProcedure
Procedure.f Mod(Value.f, Factor.f) ;Modulo function for integers and floats, too.
  ProcedureReturn Value - Int(Value / Factor) * Factor
EndProcedure
Procedure.i Sgn(Value.i) ;Returns the signum of the value.
  If Value > 0
    ProcedureReturn 1
  ElseIf Value < 0
    ProcedureReturn -1
  Else
    ProcedureReturn 0
  EndIf
EndProcedure
Procedure.i Fak(N.i)
  Protected Result.i
  
  Result = 1
  While N > 1
    Result * N
    N - 1
  Wend
  ProcedureReturn Result
EndProcedure

; Geometirc:
Procedure.f Diag(X.f, Y.f) ;Returns the square root of the sum of x and y.
  Sqr(X*X + Y*Y)
EndProcedure
Procedure.f Distance(X1.f, Y1.f, X2.f, Y2.f)
  Protected XD.f, YD.f
  
  XD = X2 - X1
  YD = Y2 - Y1
  ProcedureReturn Sqr(XD*XD + YD*YD)
EndProcedure

; Logical:
Procedure.i Is(Value.i) ;Returns 1 if the value is a true value else 0.
  If Value
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure

; Borders:
Procedure.f Min(Float.f, Minimum.f) ;Returns minimum, if the float is smaller than minimum.
  If Float => Minimum
    ProcedureReturn Float
  Else
    ProcedureReturn Minimum
  EndIf
EndProcedure
Procedure.f Max(Float.f, Maximum.f) ;Returns maximum, if the float is bigger than maximum.
  If Float <= Maximum
    ProcedureReturn Float
  Else
    ProcedureReturn Maximum
  EndIf
EndProcedure
Procedure.f Lim(Float.f, Minimum.f, Maximum.f) ;Call Min() and Max() together.
  If Float <= Minimum
    ProcedureReturn Minimum
  ElseIf Float => Maximum
    ProcedureReturn Maximum
  Else
    ProcedureReturn Float
  EndIf
EndProcedure

; Trigonometric functions (extension):
Procedure.d Cot(Angle.d)
  ProcedureReturn 1.0/Tan(Angle)
EndProcedure
Procedure.d Sec(Angle.d)
  ProcedureReturn 1.0/Cos(Angle)
EndProcedure
Procedure.d Csc(Angle.d)
  ProcedureReturn 1.0/Sin(Angle)
EndProcedure



CompilerEndIf

; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 1
; Folding = -0-
; EnableXP