;**************************************************************
;    Memory
;  
;  Authors: Josef Sniatecki
;  Copyright (c) 2010 Josef Sniatecki
;**************************************************************



CompilerIf Defined(Memory_PBI, #PB_Constant) = #False
#Memory_PBI = #True



;******************************************
; Functions
;******************************************

; String:
Procedure.s SStr(String.s, Length.i)
  If Length > 0
    ProcedureReturn Left(String, Len(String) - Length)
  ElseIf Length < 0
    ProcedureReturn Right(String, Len(String) + Length)
  Else
    ProcedureReturn String
  EndIf
EndProcedure
Procedure.s MStr(String.s, Count.i)
  ;ENGLISH:
  ;  Multiplies a string "count" times.
  
  Protected NewString.s, Index.i
  
  For Index = 1 To Count
    NewString + String
  Next
  ProcedureReturn NewString
EndProcedure
Procedure.i WriteNTString(File.i, String.s)
  ;ENGLISH:
  ;  Writes a null-terminated string into a file.
  
  WriteString(File, String)
  WriteCharacter(File, 0)
EndProcedure
Procedure.s ReadNTString(File.i)
  ;ENGLISH:
  ;  Reads a null-terminated string out of a file.
  
  Protected String.s, Character.c
  
  Repeat
    Character = ReadCharacter(File)
    If Character = 0
      ProcedureReturn String
    EndIf
    String + Chr(Character)
  ForEver
EndProcedure
Procedure.s Form(Value.i, Singular.s, Plural.s)
  ;ENGLISH:
  ;  Returns a string in the correct numeric form.
  
  If Value = -1 Or Value = 1
    ProcedureReturn Str(Value) + " " + Singular
  Else
    ProcedureReturn Str(Value) + " " + Plural
  EndIf
EndProcedure

; Depending on platform:
CompilerSelect #PB_Compiler_OS
  CompilerCase #PB_OS_Windows
    Procedure.i RAM()
     ;ENGLISH:
     ;  Returns the rest volume of RAM.
     
     Protected Info.MemoryStatus
     
     Info\dwLength = SizeOf(MemoryStatus)
     GlobalMemoryStatus_(@Info)
     ProcedureReturn Info\dwAvailPhys
    EndProcedure
    
    ; Converters for console output and -input:
    Procedure.s ConsoleToChar(String.s)
      OemToChar_(@String, @String)
      ProcedureReturn String
    EndProcedure
    Procedure.s CharToConsole(String.s)
      CharToOem_(@String, @String)
      ProcedureReturn String
    EndProcedure
CompilerEndSelect



CompilerEndIf

; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 76
; FirstLine = 43
; Folding = +-
; EnableXP