;**************************************************************
;    Memory manager
;  
;  Authors: Josef Sniatecki
;  Copyright (c) 2010 Josef Sniatecki
;**************************************************************



CompilerIf Defined(MemoryManager_PBI, #PB_Constant) = #False
#MemoryManager_PBI = #True



;******************************************
; Includes
;******************************************

IncludeFile #PB_Compiler_FilePath + "Queue.pbi"



;******************************************
; Structures
;******************************************

Structure MemoryManagerEntry
  ProcedureName.s
  *Address
  Size.i
  ID.i
EndStructure



;******************************************
; Variables
;******************************************

Global *MemoryManagerQueue.Queue
Global *MemoryManagerEntry.MemoryManagerEntry
Global MemoryManagerEnableDebugging.i = #True
Global MemoryManagerID.i



;******************************************
; Declarations
;******************************************

Declare.i MemoryManagerAllocate(Size.i)
Declare.i MemoryManagerFree(*Memory)
Declare.i MemoryManagerFreeEntry(*Entry.MemoryManagerEntry)

Declare.i MemoryManagerReset()
Declare.i MemoryManagerEnableDebugging(State.i)
Declare.i MemoryManagerCount()
Declare.i MemoryManagerDebugGarbage()



;******************************************
; Functions
;******************************************

Procedure.i MemoryManagerAllocate(Size.i)
  *MemoryManagerEntry = _New(MemoryManagerEntry)
  
  If Not *MemoryManagerQueue
    *MemoryManagerQueue = CreateQueue(@MemoryManagerFreeEntry())
  EndIf
  
  AppendQueueItem(*MemoryManagerQueue, *MemoryManagerEntry)
  
  *MemoryManagerEntry\Address = AllocateMemory(Size)
  *MemoryManagerEntry\Size    = Size
  *MemoryManagerEntry\ID      = MemoryManagerID
  
  MemoryManagerID + 1
  
  ProcedureReturn *MemoryManagerEntry\Address
EndProcedure
Procedure.i MemoryManagerFree(*Memory)
  Protected *Item.QueueItem, *Entry.MemoryManagerEntry
  
  *Item = *MemoryManagerQueue\First
  While *Item
    *Entry = *Item\Memory
    
    If *Entry\Address = *Memory
      If MemoryManagerEnableDebugging
        Debug "= FREE MEMORY ="
        Debug "  ID     : " + Str(*Entry\ID)
        Debug "  CREATOR: " + *Entry\ProcedureName
        Debug "  COUNT  : " + Str(*MemoryManagerQueue\Count - 1)
      EndIf
      
      FreeQueueItem(*MemoryManagerQueue, *Item)
      ProcedureReturn #True
    EndIf
    
    _Move(*Item, N)
  Wend
EndProcedure
Procedure.i MemoryManagerFreeEntry(*Entry.MemoryManagerEntry)
  ClearStructure(*Entry, MemoryManagerEntry)
  _Free(*Entry)
  
  ProcedureReturn #True
EndProcedure

Procedure.i MemoryManagerReset()
  ClearQueue(*MemoryManagerQueue)
  MemoryManagerID = 0
EndProcedure
Procedure.i MemoryManagerEnableDebugging(State.i)
  MemoryManagerEnableDebugging = State
EndProcedure
Procedure.i MemoryManagerCount()
  If *MemoryManagerQueue
    ProcedureReturn *MemoryManagerQueue\Count
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.i MemoryManagerDebugGarbage()
  Protected *Item.QueueItem, *Entry.MemoryManagerEntry
  
  *Item = *MemoryManagerQueue\First
  While *Item
    *Entry = *Item\Memory
    
    Debug "= LOST MEMORY ="
    Debug "  ID     : " + Str(*Entry\ID)
    Debug "  CREATOR: " + *Entry\ProcedureName
    Debug "  ADDRESS: " + Str(*Entry\Address)
    Debug "  SIZE   : " + Form(*Entry\Size, "byte", "bytes")
    
    _Move(*Item, N)
  Wend
EndProcedure



Macro AllocateMemory(MemorySize)
  MemoryManagerAllocate(MemorySize)
  *MemoryManagerEntry\ProcedureName = #PB_Compiler_Procedure
  
  If MemoryManagerEnableDebugging
    Debug "= ALLOCATE MEMORY ="
    Debug "  ID     : " + Str(*MemoryManagerEntry\ID)
    Debug "  CREATOR: " + *MemoryManagerEntry\ProcedureName
    Debug "  COUNT  : " + Str(*MemoryManagerQueue\Count)
  EndIf
EndMacro
Macro FreeMemory(Memory)
  MemoryManagerFree(Memory)
EndMacro



CompilerEndIf

; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 18
; Folding = Ay
; EnableXP