;**************************************************************
;    Queue
;  
;  Authors: Josef Sniatecki
;  Copyright (c) 2010 Josef Sniatecki
;**************************************************************
;DEUTSCH:
;  Eine doppelt verknpfte Liste fr jeden Datentyp. Vorerst als
;  Alternative zur PB-LinkedList, die nicht in Strukturen
;  eingebaut werden kann.



CompilerIf Defined(Queue_PBI, #PB_Constant) = #False
#Queue_PBI = #True



;******************************************
; Includes
;******************************************

IncludeFile #PB_Compiler_FilePath + "Standard.pbi"



;******************************************
; Constants
;******************************************

#Queue_SaveMemory = 0
#Queue_FreeMemory = 1

#Queue_CopyAddress = 0
#Queue_CopyMemory  = 1



;******************************************
; Structures
;******************************************

Structure Queue
  Count.i
  *First.QueueItem
  *Last.QueueItem
  *Destructor
EndStructure

CompilerIf Defined(QueueItem_Memory, #PB_Structure)
  Structure QueueItem Extends QueueItem_Memory
    *N.QueueItem
    *P.QueueItem
  EndStructure
CompilerElse
  Structure QueueItem
    *N.QueueItem
    *P.QueueItem
    StructureUnion
      *Memory
      *Item.QueueItem
    EndStructureUnion
  EndStructure
CompilerEndIf



;******************************************
; Functions
;******************************************

Procedure.i CreateQueue(Destructor.i = #Queue_SaveMemory)
  ;DEUTSCH:
  ;  Gibt die Adresse zu einer neuen Liste zurck.
  ;  Der Destruktor kann drei Attribute annehmen:
  ;  - #Queue_SaveMemory : Lscht nicht die Inhalte
  ;                        der Items.
  ;  - #Queue_FreeMemory : Lscht auch die Inhalte
  ;                        der Items.
  ;  - @Address : Ruft die Addrese der Prozedur
  ;               beim lschen eines Items auf.
  ;               Dabei wird als Parameter der Inhalt
  ;               als Zeiger bergeben.
  
  Protected *This.Queue
  
  *This            = _New(Queue)
  *This\Destructor = Destructor
  
  ProcedureReturn *This
EndProcedure
Procedure.i FreeQueue(*This.Queue)
  ;DEUTSCH:
  ;  Lscht alle Elemente und die Liste "this" selbst.
  
  Protected *Item.QueueItem
  
  If *This\Destructor = #Queue_FreeMemory
    While *This\First
      *Item = *This\First\N ;Auf das zweite Element verweisen.
      _Free(*This\First\Memory)
      _Free(*This\First)
      *This\First = *Item
    Wend
  ElseIf *This\Destructor
    While *This\First
      *Item = *This\First\N ;Auf das zweite Element verweisen.
      _Call(*This\Destructor, *This\First\Memory)
      _Free(*This\First)
      *This\First = *Item
    Wend
  Else
    While *This\First
      *Item = *This\First\N
      _Free(*This\First)
      *This\First = *Item
    Wend
  EndIf
  
  ProcedureReturn _Free(*This)
EndProcedure

Procedure.i ClearQueue(*This.Queue)
  ;DEUTSCH:
  ;  Lscht alle Elemente der Liste.
  
  Protected *Item.QueueItem
  
  If *This\Destructor = #Queue_FreeMemory
    While *This\First
      *Item = *This\First\N ;Auf das zweite Element verweisen.
      _Free(*This\First\Memory)
      _Free(*This\First)
      *This\First = *Item
    Wend
  ElseIf *This\Destructor
    While *This\First
      *Item = *This\First\N ;Auf das zweite Element verweisen.
      _Call(*This\Destructor, *This\First\Memory)
      FreeMemory(*This\First)
      *This\First = *Item
    Wend
  Else
    While *This\First
      *Item = *This\First\N
      _Free(*This\First)
      *This\First = *Item
    Wend
  EndIf
  
  *This\Count = 0
  *This\Last  = #Null
  
  ProcedureReturn *This
EndProcedure
Procedure.i CopyQueue(*Source.Queue, *Destination.Queue, Callback.i = #Queue_CopyMemory, *DestinationItem.QueueItem = #Null)
  Protected *SourceItem.QueueItem, *This.QueueItem
  
  *SourceItem = *Source\First
  While *SourceItem
    *This = _New(QueueItem)
    
    If *DestinationItem
      *This\P = *DestinationItem
      If *DestinationItem\N
        *This\N   = *DestinationItem\N
        *This\N\P = *This
      Else
        *Destination\Last = *This
      EndIf
      *DestinationItem\N = *This
    ElseIf *Destination\First
      *This\N            = *Source\First
      *This\N\P          = *This
      *Destination\First = *This
    Else
      *Destination\First = *This
      *Destination\Last  = *This
    EndIf
    *DestinationItem = *This
    
    If Callback = #Queue_CopyMemory
      *This\Memory = AllocateMemory(MemorySize(*SourceItem\Memory))
      CopyMemory(*SourceItem\Memory, *This\Memory, MemorySize(*SourceItem\Memory))
    ElseIf Callback
      *This\Memory = AllocateMemory(MemorySize(*SourceItem\Memory))
      _Call(Callback, *SourceItem\Memory, *This\Memory)
    Else
      *This\Memory = *SourceItem\Memory
    EndIf
    
    _Move(*SourceItem, N)
  Wend
  
  *Destination\Count + *Source\Count
  
  ProcedureReturn *Destination
EndProcedure

Procedure.i AppendQueueItem(*This.Queue, *Memory)
  ;DEUTSCH:
  ;  Fgt ein Element "*Memory" an das Ende der Liste "*This" ein.
  
  Protected *ThisItem.QueueItem
  
  *ThisItem = _New(QueueItem)
  
  If *This\Last
    *ThisItem\P   = *This\Last
    *ThisItem\P\N = *ThisItem
  Else
    *This\First = *ThisItem
  EndIf
  *ThisItem\Memory = *Memory
  
  *This\Count + 1
  *This\Last  = *ThisItem
  
  ProcedureReturn *ThisItem
EndProcedure
Procedure.i PrependQueueItem(*This.Queue, *Memory)
  ;DEUTSCH:
  ;  Fgt ein Element "*Memory" an den Anfang der Liste "*This" ein.
  
  Protected *ThisItem.QueueItem
  
  *ThisItem = _New(QueueItem)
  
  If *This\First
    *ThisItem\N   = *This\First
    *ThisItem\N\P = *ThisItem
  Else
    *This\Last = *ThisItem
  EndIf
  *ThisItem\Memory = *Memory
  
  *This\Count + 1
  *This\First  = *ThisItem
  
  ProcedureReturn *ThisItem
EndProcedure
Procedure.i InsertQueueItem(*This.Queue, *Memory, *P.QueueItem)
  Protected *ThisItem.QueueItem
  
  *ThisItem = _New(QueueItem)
  
  If *P
    *ThisItem\P = *P
    If *P\N
      *ThisItem\N = *P\N
      *P\N\P      = *ThisItem
    Else
      *This\Last = *ThisItem
    EndIf
    *P\N = *ThisItem
  ElseIf *This\First
    *This\First\P = *ThisItem
    *ThisItem\N   = *This\First
    *This\First   = *ThisItem
  Else
    *This\First = *ThisItem
    *This\Last  = *ThisItem
  EndIf
  *ThisItem\Memory = *Memory
  
  *This\Count + 1
  
  ProcedureReturn *ThisItem
EndProcedure
Procedure.i FreeQueueItem(*This.Queue, *ThisItem.QueueItem)
  Protected *NextItem.QueueItem
  
  *NextItem = *ThisItem\N
  
  If *ThisItem\N
    *ThisItem\N\P = *ThisItem\P
  Else
    *This\Last = *ThisItem\P
  EndIf
  If *ThisItem\P
    *ThisItem\P\N = *ThisItem\N
  Else
    *This\First = *ThisItem\N
  EndIf
  
  If *This\Destructor = #Queue_FreeMemory
    _Free(*ThisItem\Memory)
  ElseIf *This\Destructor
    _Call(*This\Destructor, *ThisItem\Memory)
  EndIf
  _Free(*ThisItem)
  
  *This\Count - 1
  
  ProcedureReturn *NextItem
EndProcedure
Procedure.i FreeLastQueueItem(*This.Queue)
  Protected *LastItem.QueueItem
  
  *LastItem = *This\Last
  If *LastItem\P
    *LastItem\P\N = #Null
    *This\Last    = *LastItem\P
  Else
    *This\First = #Null
    *This\Last  = #Null
  EndIf
  
  If *This\Destructor = #Queue_FreeMemory
    _Free(*LastItem\Memory)
  ElseIf *This\Destructor
    _Call(*This\Destructor, *LastItem\Memory)
  EndIf
  _Free(*LastItem)
  
  *This\Count - 1
  
  ProcedureReturn *This\Last
EndProcedure
Procedure.i FreeFirstQueueItem(*This.Queue)
  ;DEUTSCH:
  ;  Lscht das erste Element aus der Liste "*This".
  
  Protected *FirstItem.QueueItem
  
  *FirstItem = *This\First
  If *FirstItem\N
    *FirstItem\N\P = #Null
    *This\First    = *FirstItem\N
  Else
    *This\First = #Null
    *This\Last  = #Null
  EndIf
  
  If *This\Destructor = #Queue_FreeMemory
    _Free(*FirstItem\Memory)
  ElseIf *This\Destructor
    _Call(*This\Destructor, *FirstItem\Memory)
  EndIf
  _Free(*FirstItem)
  
  *This\Count - 1
  
  ProcedureReturn *This\First
EndProcedure

Procedure.i PopQueueItem(*This.Queue)
  Protected *Item.QueueItem, *Memory
  
  *Item = *This\Last
  If *Item\P
    *Item\P\N  = #Null
    *This\Last = *Item\P
  Else
    *This\First =  #Null
    *This\Last  = #Null
  EndIf
  
  *Memory = *Item\Memory
  _Free(*Item)
  
  *This\Count - 1
  ProcedureReturn *Memory
EndProcedure
Procedure.i SelectQueueItem(*This.Queue, Index.i, *ResultMemory.Integer = 0)
  Protected *Item.QueueItem
  
  If *This\Count
    If Index < *This\Count / 2
      *Item = *This\First
      While Index
        _Move(*Item, N)
        Index - 1
      Wend
    ElseIf Index <= *This\Count
      *Item = *This\Last
      Index = *This\Count - Index - 1
      While Index
        _Move(*Item, P)
        Index - 1
      Wend
    EndIf
    
    If *ResultMemory
      *ResultMemory\I = *Item\Memory
    EndIf
    
    ProcedureReturn *Item
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i RandomQueueItem(*This.Queue, *ResultMemory.Integer = 0)
  Protected Index.i
  Protected *Item.QueueItem
  
  Index = Random(*This\Count - 1)
  
  If Index < *This\Count / 2
    *Item = *This\First
    While Index
      If *Item
        _Move(*Item, N)
      Else
        ProcedureReturn #False
      EndIf
      Index - 1
    Wend
  Else
    *Item = *This\Last
    Index = *This\Count - Index - 1
    While Index
      If *Item
        _Move(*Item, P)
      Else
        ProcedureReturn #False
      EndIf
      Index - 1
    Wend
  EndIf
  
  If *ResultMemory
    *ResultMemory\I = *Item\Memory
  EndIf
  ProcedureReturn *Item
EndProcedure

Procedure.i MemoryQueue(*This.Queue, *Result.Queue = #Null)
  Protected *Item.QueueItem
  
  If Not *Result
    *Result = CreateQueue()
  EndIf
  
  *Item = *This\First
  While *Item
    AppendQueueItem(*Result, *Item\Item\Memory)
    _Move(*Item, N)
  Wend
  
  ProcedureReturn *Result
EndProcedure
Procedure.i SubtractQueue(*This.Queue, *ToSubtract.Queue, *Result.Queue = #Null)
  ;GERMAN:
  ;  Lscht alle Elemente, dessen Addresse in der Liste "ToSubtract" enthalten
  ;  sind.
  
  Protected *Item.QueueItem, *OtherItem.QueueItem, DoAdd.i
  
  If Not *Result
    *Result = CreateQueue()
  EndIf
  
  *Item = *This\First
  While *Item
    DoAdd = #True
    *OtherItem = *ToSubtract\First
    While *OtherItem
      If *Item = *OtherItem\Item
        DoAdd = #False
        Break
      EndIf
      _Move(*OtherItem, N)
    Wend
    If DoAdd
      AppendQueueItem(*Result, *Item\Memory)
    EndIf
    _Move(*Item, N)
  Wend
  
  ProcedureReturn *Result
EndProcedure
Procedure.i SortQueue(*This.Queue, *Callback)
  ;GERMAN:
  ;  Sortiert die Liste nach dem Bubblesort-Algorithmus.
  
  Protected N.i, Swapped.i
  Protected *Item.QueueItem, *OtherItem.QueueItem, *TempItem.QueueItem
  
  N = *This\Count
  
  Repeat
    Swapped = #False
    *Item = *This\First
    While *Item
      *OtherItem = *Item\N
      If *OtherItem
        If Not _Call(*Callback, *Item\Memory, *OtherItem\Memory)
          ; Swap items:
          *OtherItem\P = *Item\P
          *Item\P      = *OtherItem
          *Item\N      = *OtherItem\N
          *OtherItem\N = *Item
          If *OtherItem\P
            *OtherItem\P\N = *OtherItem
          Else
            *This\First = *OtherItem
          EndIf
          If *Item\N
            *Item\N\P = *Item
          Else
            *This\Last = *Item
          EndIf
          Swapped = #True
        EndIf
      EndIf
      _Move(*Item, N)
    Wend
    N - 1
  Until Not (Swapped And N => 1)
  
  ProcedureReturn *This
EndProcedure




CompilerEndIf

; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 82
; FirstLine = 62
; Folding = ----
; EnableXP
; Executable = ..\Projekte\Operno 5\Operno.exe