;**************************************************************
;          ______  ______          ______  _______  _
;         / __  / / __  /         / ____/ /__  __/ / /
;        / /_/ / / /_/ /  ___    / /___     / /   / /
;       / ____/ / __ |   |___|  /__   /    / /   / /
;      / /     / /_/ /         ____/ /    / /   / /____
;     /_/     /_____/         /_____/    /_/   /______/
;
;  Author: Josef Sniatecki
;  Copyright (c) 2010 Josef Sniatecki
;**************************************************************
;DEUTSCH:
;  Dies ist die Hauptdatei aller Includes. In dieser sind oft
;  verwendete Konstanten, Makros und Funktionen enthalten.



CompilerIf Defined(Standard_PBI, #PB_Constant) = #False
#Standard_PBI = #True



;******************************************
; Macros
;******************************************
;DEUTSCH:
;  Vorschlag: Diese Makros als Schlsselwrter
;             definieren.

; Capsuling:
Macro UseBlock
  Repeat
EndMacro
Macro EndBlock
  Until #False
EndMacro

; Allocators and destructors:
Macro _New(Object)
  AllocateMemory(SizeOf(Object))
EndMacro
Macro _Dim(Size)
  AllocateMemory(Size)
EndMacro
Macro _Free(Memory)
  FreeMemory(Memory)
EndMacro

; Misc macros:
Macro _Move(Pointer, Field)
  Pointer = Pointer\Field
EndMacro
Macro _Sz(Object)
  SizeOf(Object)
EndMacro
Macro _Inv(Variable)
  Variable = -Variable
EndMacro
Macro _Min(Variable, Minimum)
  If Variable < Minimum
    Variable = Minimum
  EndIf
EndMacro
Macro _Max(Variable, Maximum)
  If Variable > Maximum
    Variable = Maximum
  EndIf
EndMacro
Macro _Lim(Variable, Minimum, Maximum)
  If Variable > Maximum
    Variable = Maximum
  ElseIf Variable < Minimum
    Variable = Maximum
  EndIf
EndMacro

Macro _Call
  CallFunctionFast
EndMacro

Macro _Q
"
EndMacro

Macro _LowWord(Value)
  (Value & $FFFF)
EndMacro
Macro _HighWord(Value)
  (Value >> 16)
EndMacro



;******************************************
; Constants
;******************************************

; PB datatype borders (thanks to STARGATE (user at http://www.pure-board.de/)):
#PB_MinByte    = -$80
#PB_MaxByte    =  $7F
#PB_MinAscii   =  $00
#PB_MaxAscii   =  $FF

#PB_MinWord    = -$8000
#PB_MaxWord    =  $7FFF
#PB_MinUnicode =  $0000
#PB_MaxUnicode =  $FFFF

CompilerIf SizeOf(Character) = SizeOf(Ascii)
  #PB_MinCharacter = #PB_MinAscii
  #PB_MaxCharacter = #PB_MaxAscii
CompilerElse
  #PB_MinCharacter = #PB_MinUnicode
  #PB_MaxCharacter = #PB_MaxUnicode
CompilerEndIf

#PB_MinLong = -$80000000
#PB_MaxLong =  $7FFFFFFF
#PB_MinQuad = -$8000000000000000
#PB_MaxQuad =  $7FFFFFFFFFFFFFFF

CompilerIf SizeOf(Integer) = SizeOf(Long)
  #PB_MinInteger = #PB_MinLong
  #PB_MaxInteger = #PB_MaxLong
CompilerElse
  #PB_MinInteger = #PB_MinQuad
  #PB_MaxInteger = #PB_MaxQuad
CompilerEndIf

; Stats:
#True     =  1
#False    =  0
#Enabled  =  1
#Disabled =  0
#Any      = -1
#Endless  = -1
#Error    =  0
#Done     =  1
#None     = -1

; Significant options:
#Positive =  1
#Neutral  =  0
#Negative = -1
#Great    =  1
#Equal    =  0
#Less     = -1
#Next     =  1
#Previous = -1

; 2D-Directions:
Enumeration
  #Down
  #Up
  #Right
  #Left
EndEnumeration

; Characters:
#N  = Chr(13) + Chr(10) ;Newline
#S  = Chr(10)           ;Seperator
#Q  = Chr(34)           ;Double quote
#T  = Chr(9)            ;Tab
#TM = Chr(153)          ;Trade mark
#C  = Chr(169)          ;Copy
#R  = Chr(174)          ;Right

; String:
#UABC    = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
#LABC    = "abcdefghijklmnopqrstuvwxyz"
#Numbers = "0123456789"

; Types:
Enumeration
  #Variant
  #Byte
  #Character
  #Word
  #Long
  #Integer
  #Float
  #Quad
  #Double
  #String
EndEnumeration



;******************************************
; Structures
;******************************************

Structure All
  StructureUnion
    I.i
    F.f
    Q.q
    D.d
    C.c
    B.b
    *P.All
  EndStructureUnion
EndStructure
Structure V2
  StructureUnion
    X.f
    Width.f
  EndStructureUnion
  StructureUnion
    Y.f
    Height.f
  EndStructureUnion
EndStructure
Structure V3
  StructureUnion
    X.f
    Width.f
  EndStructureUnion
  StructureUnion
    Y.f
    Height.f
  EndStructureUnion
  StructureUnion
    Z.f
    Depth.f
  EndStructureUnion
EndStructure
Structure DLLString
  Length.i
  String.s
EndStructure



;******************************************
; Procedures
;******************************************

Procedure.s StrFN(Float.f)
  Protected String.s, *Pointer.Character
  
  String = StrF(Float)
  
  *Pointer = @String + Len(String) - 1
  While *Pointer\C = '0'
    *Pointer - 1
  Wend
  If *Pointer\C = '.'
    *Pointer - 1
  EndIf
  
  ProcedureReturn Left(String, *Pointer - @String + 1)
EndProcedure
Procedure.s StrDN(Double.d)
  Protected String.s, *Pointer.Character
  
  String = StrD(Double)
  
  *Pointer = @String + Len(String) - 1
  While *Pointer\C = '0'
    *Pointer - 1
  Wend
  If *Pointer\C = '.'
    *Pointer - 1
  EndIf
  
  ProcedureReturn Left(String, *Pointer - @String + 1)
EndProcedure

Procedure.i FPS()
  Static Time.i
  Protected FPS.i, CurrentTime.i
  
  If Time
    CurrentTime = ElapsedMilliseconds()
    FPS         = CurrentTime-Time
    Time        = CurrentTime
  Else
    Time = ElapsedMilliseconds()
    FPS  = 0
  EndIf
  ProcedureReturn FPS
EndProcedure
Procedure.i OptimizeFPS(StandardTime.i = 16)
  Static Time.i
  Protected Difference.i, CurrentTime.i
  
  If Time
    CurrentTime = ElapsedMilliseconds()
    Difference  = CurrentTime-Time
    Time        = CurrentTime
  Else
    Time       = ElapsedMilliseconds()
    Difference = 0
  EndIf
  If Difference < StandardTime
    Delay(StandardTime - Difference)
  EndIf
EndProcedure



;******************************************
; Includes
;******************************************

IncludeFile "Memory.pbi"
IncludeFile "Math.pbi"
IncludeFile "System.pbi"



CompilerEndIf

; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 5
; Folding = ---h
; EnableXP